package kr.gmtc.eyegw;

import java.time.ZoneId;
import java.util.TimeZone;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.autoconfigure.orm.jpa.HibernateJpaAutoConfiguration;
import org.springframework.boot.context.ApplicationPidFileWriter;
import org.springframework.boot.context.event.ApplicationEnvironmentPreparedEvent;
import org.springframework.boot.system.ApplicationHome;
import org.springframework.context.ApplicationListener;
import org.springframework.scheduling.annotation.EnableScheduling;

import gmt.logger.GmtLogManager;

@SpringBootApplication
@EnableAutoConfiguration(exclude={DataSourceAutoConfiguration.class, HibernateJpaAutoConfiguration.class})
@EnableScheduling
public class GmtApplication {

	 @Value("${root}")
	 String path;
	 
	public static void main(String[] args) {
		
		ApplicationHome home = new ApplicationHome(GmtApplication.class);
		String root = home.getDir().getPath();
		
		System.setProperty("user.dir", root);
		
		Thread.currentThread().setName("JVM - Main");
		
		TimeZone.setDefault(TimeZone.getTimeZone(ZoneId.of("UTC")));
		
		SpringApplication springApplication = new SpringApplication(GmtApplication.class);
		springApplication.addListeners(new LogPathSetter());
		springApplication.addListeners(new ApplicationPidFileWriter("./applicaion.pid"));
		springApplication.run(args);

	}
	
}

class LogPathSetter implements ApplicationListener<ApplicationEnvironmentPreparedEvent> {

	   @Override
	   public void onApplicationEvent(ApplicationEnvironmentPreparedEvent applicationEvent) {
	      GmtLogManager.setpath(applicationEvent.getEnvironment().getProperty("root"));
	   }
	}