package kr.gmtc.eyegw.db;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.type.JdbcType;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.EnableTransactionManagement;

@Mapper
@Configuration
@EnableTransactionManagement
@ConditionalOnProperty("db.db1.use")
@Component("DB1Connection")
public class DB1Connection {
	
	@Bean(name="DB1DataSource")
	@ConfigurationProperties(prefix = "db.db1.datasource")
	public DataSource db1DataSource() {
		return DataSourceBuilder.create().build();
	}
	
	@Bean(name="DB1SqlSessionFactory")
	public SqlSessionFactory db1SqlSessionFactory(
			@Qualifier("DB1DataSource") DataSource dataSource,
			ApplicationContext applicationContext) throws Exception {
		
		SqlSessionFactoryBean sqlSessionFactoryBean = new SqlSessionFactoryBean();
		
		// DataSource 지정
		sqlSessionFactoryBean.setDataSource(dataSource);
		
		// sqlmap 경로 지정
		sqlSessionFactoryBean.setMapperLocations(applicationContext.getResources("classpath:/kr/gmtc/eyegw/mapper/dao1/DB1.xml"));
		
		// 캐시 사용 여부
		sqlSessionFactoryBean.getObject().getConfiguration().setCacheEnabled(false);
		
		// JDBC null 데이터 타입 지정
		sqlSessionFactoryBean.getObject().getConfiguration().setJdbcTypeForNull(JdbcType.NULL);
		
		
		
		return sqlSessionFactoryBean.getObject();
	}
	
	@Bean(name = "DB1Session")
	public SqlSessionTemplate session(
			@Qualifier("DB1SqlSessionFactory") SqlSessionFactory sqlSessionFactory) {
		return new SqlSessionTemplate(sqlSessionFactory);
	}
	
	@Bean(name = "DB1BatchSession")
	public SqlSessionTemplate batchSession(
			@Qualifier("DB1SqlSessionFactory") SqlSessionFactory sqlSessionFactory) {
		return new SqlSessionTemplate(sqlSessionFactory, ExecutorType.BATCH);
	}
	
	@Resource(name = "DB1Session")
	public SqlSessionTemplate session;
	
	@Resource(name = "DB1BatchSession")
	public SqlSessionTemplate batchSession;
	
	public boolean isAlive() {
		session.selectOne("kr.gmtc.eyegw.mapper.dbo1.DB1.ping");
		return true;
	}
			
	
}
