package kr.gmtc.eyegw.frame;

import gmt.common.type.LogLevelType;
import gmt.io.client.vo.ClientInfoVO;
import gmt.io.server.vo.ServerInfoVO;
import gmt.io.server.vo.ServiceClientVO;
import gmt.logger.GmtLogManager;
import kr.gmtc.eyegw.controller.MainController;

public class StatusReporter {
	private final CustomThread reporter;
	private final MainController controller;
	private final GmtLogManager logger;
	
	public StatusReporter(MainController controller, long reportInterval) {
		reporter = new CustomThread("stateReporter", controller, reportInterval, this::reportState, null, false);
		this.controller = controller;
		this.logger = GmtLogManager.getInstance();
	}
	
	/** 상태 보고 스레드 */
	private void reportState() {
		try {
			StringBuilder logContent = new StringBuilder();
			logContent.append("\n=================== Monitoring info ===================");
			logContent.append("\n");
			logContent.append(String.format("1. Total RcvQ  : %d/%d", controller.getCurrnetRecvQCount(), 100000));
			logContent.append("\n");
			logContent.append(String.format("2. Total SndQ  : %d/%d", controller.getCurrnetSendQCount(), 100000));
			int cnt;
			if (controller.clientAccess) {
				logContent.append("\n3. Client Monitor Info(s)");
				cnt = 0;
				for (ClientInfoVO clientInfo: controller.clientInfoList) {
					String infoStr = String.format("%d. [%s:%d] %s: Rcv - %d, Snd - %d, QCnt - %d/%d",
							cnt++,
							clientInfo.getIp(),
							clientInfo.getPort(),
							clientInfo.getName(),
							clientInfo.getMonitorInfo().getRecvMonitorCount(),
							clientInfo.getMonitorInfo().getSendMonitorCount(),
							clientInfo.getMonitorInfo().getSendQCount(),
							clientInfo.getMonitorInfo().getSendQBufferCount());
					logContent.append("\n\t");
					logContent.append(infoStr);
				}
			} else {
				logContent.append("\n3. Client Not Used.");
			}
			
			if (controller.serverAccess) {
				logContent.append("\n4. Server Monitor Info(s)");
				cnt = 0;
				for (ServerInfoVO serverInfo: controller.serverInfoList) {
					String infoStr = String.format("%d. [:%d] %s: SC cnt - %d, QCnt - %d/%d",
							cnt++,
							serverInfo.getPort(),
							serverInfo.getName(),
							serverInfo.getServiceClientCount(),
							serverInfo.getSndQcount(),
							serverInfo.getMaxSndQCount());
					logContent.append("\n\t");
					logContent.append(infoStr);
				}
				logContent.append("\n5. Service Client Monitor Info(s)");

				cnt = 0;
				for (ServiceClientVO serverClientVO: controller.serverScInfoList.values()) {
					String infoStr = String.format("%d. [:%d <- %s:%d] %s: Rcv - %d, Snd - %d, RcvByte - %d, SndByte - %d, Qcnt - %d/%d",
							cnt++,
							serverClientVO.getServerPort(),
							serverClientVO.getIp(),
							serverClientVO.getServicePort(),
							serverClientVO.getName(),
							serverClientVO.getMonitorInfoVO().getRecvMonitorCount(),
							serverClientVO.getMonitorInfoVO().getSendMonitorCount(),
							serverClientVO.getMonitorInfoVO().getRecvMonitorByteSize(),
							serverClientVO.getMonitorInfoVO().getSendMonitorByteSize(),
							serverClientVO.getMonitorInfoVO().getSendQCount(),
							serverClientVO.getMonitorInfoVO().getSendQBufferCount());
					logContent.append("\n\t");
					logContent.append(infoStr);
				}
			} else {
				logContent.append("\n\t4. Server Not Used.");
				logContent.append("\n\t5. Service Client Not Used.");
			}
			logContent.append("\n=======================================================");

			logger.writeLevelLog("[reportState] Report Status:" + logContent.toString(), LogLevelType.LOG_INFO, "AllLog");
		} catch (Exception e) {
			logger.writeLevelLog("[reportState] Unknown Exception Occur. " + e.getMessage(), LogLevelType.LOG_ERROR, "AllLog");
		}
	}
	
	public void start() {
		reporter.start();
	}
	
	public void stop() {
		reporter.gracefulStop();
	}
	
}